@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from argparse import ArgumentParser
-from typing import List, Optional, Dict
+from typing import List, Optional, Dict, Tuple
 
 from numpy.random import RandomState
 
@@ -38,14 +38,14 @@ class Agent(ABC):
             args: List[str],
             random_state: RandomState,
             pi: Optional[Policy]
-    ) -> List:
+    ) -> Tuple[List['Agent'], List[str]]:
         """
-        Initialize a list of agents from arguments.
+        Initialize a agents from arguments.
 
         :param args: Arguments.
         :param random_state: Random state.
         :param pi: Policy.
-        :return: List of agents.
+        :return: 2-tuple of a list of agents and a list of unparsed arguments.
         """
 
     def reset_for_new_run(
@@ -53,9 +53,9 @@ class Agent(ABC):
             state: State
     ):
         """
-        Reset the agent to a state prior to any learning.
+        Reset the agent for a new run.
 
-        :param state: New state.
+        :param state: Initial state.
         """
 
         self.most_recent_action = None
@@ -81,7 +81,7 @@ class Agent(ABC):
 
     def act(
             self,
-            t
+            t: int
     ) -> Action:
         """
         Request an action from the agent.
@@ -129,7 +129,6 @@ class Agent(ABC):
 
         :param r: Reward.
         """
-        pass
 
     def __init__(
             self,
@@ -154,7 +153,7 @@ class Agent(ABC):
 
     def __str__(
             self
-    ):
+    ) -> str:
         """
         Return name.
 
@@ -176,11 +175,13 @@ class Human(Agent):
             args: List[str],
             random_state: RandomState,
             pi: Optional[Policy]
-    ) -> List:
+    ) -> Tuple[List[Agent], List[str]]:
         """
         Not implemented.
         """
 
+        raise NotImplementedError()
+
     def __act__(
             self,
             t: int
@@ -245,6 +246,6 @@ class Human(Agent):
             random_state=None
         )
 
-        # TODO:  This is a hack to make the human agent compatible with tabular methdods, which request state
+        # TODO:  This is a hack to make the human agent compatible with tabular methods, which request state
         # identifiers during operation.
         self.pi = TabularPolicy(None, None)
