@@ -28,7 +28,7 @@ class Gridworld(ModelBasedMdpEnvironment):
     def example_4_1(
             random_state: RandomState,
             T: Optional[int]
-    ):
+    ) -> 'Gridworld':
         """
         Construct the Gridworld for Example 4.1.
 
@@ -231,7 +231,7 @@ class GridworldFeatureExtractor(StateActionInteractionFeatureExtractor):
             cls,
             args: List[str],
             environment: Gridworld
-    ) -> Tuple[StateActionInteractionFeatureExtractor, List[str]]:
+    ) -> Tuple[FeatureExtractor, List[str]]:
         """
         Initialize a feature extractor from arguments.
 
